@@ -61,7 +61,8 @@ impl Connection {
     {
         self.buffer.resize(size, 0);
         self.raw_conn.read(&mut self.buffer).await?;
-        Ok(from_bytes(&self.buffer)?)
+        // Ignore any trailing bytes to be forward compatible
+        Ok(from_bytes(&self.buffer)?.0)
     }
 
     /// Return size of the response.
@@ -79,8 +80,9 @@ impl Connection {
         T: Deserialize<'a>,
     {
         self.buffer.resize(size, 0);
+        // Ignore any trailing bytes to be forward compatible
         match self.raw_conn.try_read(&mut self.buffer)? {
-            Some(_) => Ok(Some(from_bytes(&self.buffer)?)),
+            Some(_) => Ok(Some(from_bytes(&self.buffer)?.0)),
             None => Ok(None),
         }
     }
